@@ -15,7 +15,7 @@
  *
  */
 /*
- *  Copyright (C) 2000-2015 Thomas Habets <thomas@habets.se>
+ *  Copyright (C) 2000-2016 Thomas Habets <thomas@habets.se>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -219,21 +219,21 @@ drop_fs_root()
         errno = 0;
         if (!(pw = getpwnam(chroot_user))) {
                 if (verbose) {
-                        printf("arping: getpwnam(%s): %s",
+                        printf("arping: getpwnam(%s): %s\n",
                                chroot_user, strerror(errno));
                 }
                 return;
         }
         if (chdir(pw->pw_dir)) {
                 if (verbose) {
-                        printf("arping: chdir(%s): %s",
+                        printf("arping: chdir(%s): %s\n",
                                pw->pw_dir, strerror(errno));
                 }
                 return;
         }
         if (chroot(pw->pw_dir)) {
                 if (verbose) {
-                        printf("arping: chroot(%s): %s",
+                        printf("arping: chroot(%s): %s\n",
                                pw->pw_dir, strerror(errno));
                 }
                 return;
@@ -834,6 +834,13 @@ static void
 pingmac_send(uint16_t id, uint16_t seq)
 {
 	static libnet_ptag_t icmp = 0, ipv4 = 0,eth=0;
+
+        // Padding size chosen fairly arbitrarily.
+        // Without this padding some systems (e.g. Raspberry Pi 3
+        // wireless interface) failed. dmesg said:
+        //   arping: packet size is too short (42 <= 50)
+        const char padding[16] = {0};
+
 	int c;
 
 	if (-1 == (icmp = libnet_build_icmpv4_echo(ICMP_ECHO, /* type */
@@ -841,8 +848,8 @@ pingmac_send(uint16_t id, uint16_t seq)
 						   0, /* checksum */
 						   id, /* id */
 						   seq, /* seq */
-						   NULL, /* payload */
-						   0, /* payload len */
+						   padding, /* payload */
+						   sizeof padding, /* payload len */
 						   libnet,
 						   icmp))) {
 		fprintf(stderr, "libnet_build_icmpv4_echo(): %s\n",
@@ -918,6 +925,13 @@ static void
 pingip_send()
 {
 	static libnet_ptag_t arp=0,eth=0;
+
+        // Padding size chosen fairly arbitrarily.
+        // Without this padding some systems (e.g. Raspberry Pi 3
+        // wireless interface) failed. dmesg said:
+        //   arping: packet size is too short (42 <= 50)
+        const char padding[16] = {0};
+
 	if (-1 == (arp = libnet_build_arp(ARPHRD_ETHER,
 					  ETHERTYPE_IP,
 					  ETH_ALEN,
@@ -927,8 +941,8 @@ pingip_send()
 					  (uint8_t*)&srcip,
 					  unsolicited ? (uint8_t*)ethxmas : (uint8_t*)ethnull,
 					  (uint8_t*)&dstip,
-					  NULL,
-					  0,
+					  padding,
+					  sizeof padding,
 					  libnet,
 					  arp))) {
 		fprintf(stderr, "arping: libnet_build_arp(): %s\n",
@@ -1645,7 +1659,11 @@ arping_main(int argc, char **argv)
         }
 
         if (display == DOT) {
-                setvbuf(stdout, NULL, _IONBF, 0);
+                if (0 != setvbuf(stdout, NULL, _IONBF, 0)) {
+                        fprintf(stderr,
+                                "arping: setvbuf(stdout, NULL, IONBF, 0): %s\n",
+                                strerror(errno));
+                }
         }
 
         if (finddup && maxcount == -1) {
